package cmd

import (
	"context"
	"fmt"
	"os"
	"syscall"

	"orus.io/orus-io/rednerd/rendering"
	"orus.io/orus-io/rednerd/utils"
)

type VarlistCmd struct {
	Template Template `group:"Template"`
}

func (cmd VarlistCmd) Execute(args []string) error {
	if len(args) == 1 {
		cmd.Template.DataFile = args[0]
	}
	if err := engineOptions.SetupEngines(log); err != nil {
		return err
	}
	template, err := cmd.Template.ToModelTemplate()
	if err != nil {
		return err
	}

	ctx, cancel := utils.CtxWithSignal(context.Background(), os.Interrupt, syscall.SIGQUIT, syscall.SIGTERM)
	defer cancel()
	fmt.Println(template.Language)
	t, err := rendering.DefaultTemplateEngineRegistry.Load(ctx, template)
	if err != nil {
		return err
	}
	varlist, err := t.Varlist()
	if err != nil {
		return err
	}
	for _, v := range varlist {
		fmt.Println("-", v)
	}

	return nil
}
