package wpd_test

import (
	"context"
	"os"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/engines/wpd"
	"orus.io/orus-io/rednerd/testutils"
)

func WPDSubTest(ctx context.Context, t *testing.T, para string, wpd WPD, f *os.File) {
	t.Helper()

	id, err := wpd.Submit(ctx, "", []byte(para))
	require.NoError(t, err)

	rid, c, err := wpd.NextResult()
	require.NoError(t, err)
	assert.Equal(t, id, rid)
	assert.NotZero(t, len(c))

	_, err = f.Write(c)
	require.NoError(t, err)
}

func TestRealWpd(t *testing.T) {
	log := testutils.GetLogger(t)
	p, err := wpd.NewRealProcess()

	require.NoError(t, err)
	ctx, cancel := context.WithTimeout(context.Background(), time.Second*5)
	defer cancel()

	wpd, err := wpd.NewProcessWPD(p, log)
	require.NoError(t, err)

	fileName := t.Name() + ".pdf"
	require.NoError(t, os.Remove(fileName))
	f, err := os.Create(fileName)
	require.NoError(t, err)

	WPDSubTest(ctx, t, "", wpd, f)
	WPDSubTest(ctx, t, "<p>A small paragraph</p>", wpd, f)
	WPDSubTest(ctx, t, "\u2028", wpd, f)
	//nolint:misspell
	WPDSubTest(ctx, t, `SASU au capital de 7200 â‚¬ SiÃ¨ge social : 141 RUE DE VERDUN 76600 LE HAVRE Modification au RCS de LE HAVRE 851162735

Par dÃ©cision AssemblÃ©e GÃ©nÃ©rale Extraordinaire du 08/10/2021, il a Ã©tÃ© dÃ©cidÃ© dâ€™Ã©tendre lâ€™objet social aux activitÃ©s suivantes : Sous louer tout ou partie des lieux louÃ©s au 141 rue de verdun 76600 Le Havre pour une activitÃ© de restauration de plats Ã  emporter ou Ã  livrer. â€¨`, wpd, f)

	WPDSubTest(ctx, t, " ", wpd, f)
	assert.NoError(t, p.Stop())
}
