package apitester

import (
	"io"
	"net/http"

	"github.com/k3a/html2text"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
)

// CreateTemplate creates a new template associated to an Account.
func (tester *APITester) CreateTemplate(
	username string,
	body, response *models.Template,
) {
	apitest := tester.APITest("template_create")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Post("/api/v1/template/" + username).
		JSON(body).
		Expect(tester.t).
		Status(http.StatusCreated).
		End()
	r.JSON(response)
}

// UpdateTemplate updates an existing template associated to an Account.
func (tester *APITester) UpdateTemplate(
	username, templateName string,
	body, response *models.Template,
) {
	apitest := tester.APITest("template_update")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Put("/api/v1/template/" + username + "/" + templateName).
		JSON(body).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(response)
}

// Render renders some data and returns a document.
func (tester *APITester) Render(
	data *models.RenderRequest,
	result *[]*models.Document,
) {
	apitest := tester.APITest("render")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Post("/api/v1/render/").
		JSON(data).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(result)
}

// Renderlogs gives us a csv document containing the
// Rendering data for invoicing.
func (tester *APITester) RenderLogs(
	period string,
) string {
	apitest := tester.APITest("renderlogs " + period)
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Get("/api/v1/renderlogs/" + period).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	b, err := io.ReadAll(r.Response.Body)
	require.NoError(tester.t, err)

	return string(b)
}

// Varlist gives us the variables inside a template..
func (tester *APITester) Varlist(tmpl *models.Template) []string {
	apitest := tester.APITest("varlist")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Post("/api/v1/varlist").
		JSON(tmpl).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	var varList []string
	r.JSON(&varList)

	return varList
}

func (tester *APITester) ListEngines() string {
	apitest := tester.APITest("list_engines")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Get("/docs/engines").
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	b, err := io.ReadAll(r.Response.Body)
	require.NoError(tester.t, err)
	engines := html2text.HTML2Text(string(b))

	return engines
}
